package HTFeed::Config;

use warnings;
use strict;
use YAML::AppConfigFork;
use Carp;
use File::Basename;
use File::Spec;
use Cwd qw(realpath);

use base qw(Exporter);
our @EXPORT = qw(get_config);
our @EXPORT_OK = qw(set_config);

##################################################
#               HTFeed directories               #
#  This block is generated by make, do not edit  #
#  Override these in etc/feed/config if needed   #
##################################################
my $feed_etc = "/usr/local/etc/feed"; # FEED_ETC
my $feed_bin = "/usr/local/bin"; # FEED_BIN
my $feed_var = "/usr/local/var/feed"; # FEED_VAR

my ($vstring,$full_version_string,$production_ok) = (undef,undef,undef); # HTFEED_VERSION_STRINGS
############ End HTFeed directories ##############

my $config;

init();

## TODO: make get_config die on failed top level requests

sub init{
    # find lib directory
    my $feed_lib;
    my $feed_lib_parent;
    {
        my $this_module = __PACKAGE__ . '.pm';
        $this_module =~ s/::/\//g;
        my $path_to_this_module = dirname($INC{$this_module});
        $feed_lib = realpath "$path_to_this_module/..";
        $feed_lib_parent = realpath "$path_to_this_module/../..";
    }

    # find bin, etc and var if they aren't predefined
    $feed_etc ||= "$feed_lib_parent/etc";
    $feed_bin ||= "$feed_lib_parent/bin";
    $feed_var ||= "$feed_lib_parent/var";

    # generate a simple base config
    my $base_config = <<"baseyaml";
---
feed_etc: $feed_etc
feed_lib: $feed_lib
feed_bin: $feed_bin
feed_var: $feed_var
baseyaml
    $config = YAML::AppConfigFork->new(string => $base_config);

    my $config_dir = "$feed_etc/config";

    # load config files
    my $filename;
    eval{
        foreach my $config_file (sort(glob("$config_dir/*.yml"))){
			$filename = $config_file;
            if($config){
                $config->merge(file => $config_file);
            } else {
                $config = YAML::AppConfigFork->new(file => $config_file);
            }
        }
    };
    if ($@){ die ("loading $filename failed: $@"); }

#    # add feed_app_root to config
#    unless (defined $config->{feed_app_root}){
#        $config->set('feed_home',$feed_app_root);
#    }
    
    ## TODO: check file validity, can't do this until we establish what the file will look like
    ## TODO: test script to dig out all the config vars and check against config file
}

sub get_config{
    # drill down to the leaf
    my $cursor = $config->get(shift @_);
    foreach my $hashlevel (@_) {
        # die if we try to traverse the tree past a leaf
        croak( sprintf("%s is not in the config file", join('=>',@_)) ) if (!ref($cursor));
        $cursor = $cursor->{$hashlevel};
        
        # die if we try to traverse the tree where no path exists
        croak( sprintf("%s is not in the config file", join('=>',@_)) ) if (not defined $cursor);
    }
    return $cursor;
}

# this probably shouldn't be used in production, but will be quite helpful in test scripts
sub set_config{
    my $setting = shift;
    my $leaf = pop;

    if(!@_) {
        $config->set($leaf,$setting);
    } else {
        my $topkey = shift @_;
        my $topval = $config->get($topkey);
        my $cursor = $topval;
        foreach my $hashlevel (@_) {
            $cursor = $cursor->{$hashlevel};
            croak( sprintf("%s is not in the config file", join('=>',@_,$leaf)) ) if (! ref($cursor));
        }
        $cursor->{$leaf} = $setting;
        $config->set($topkey,$topval);
    }

    return 1;
}

sub _dump {
    print $config->dump();
}

1;

__END__

=head1 NAME

HTFeed::Config - Manage HTFeed configuration settings

=head1 SYNOPSIS

Get and Set methods for interaction with config.yml files

=head1 DESCRIPTION

Config.pm provides the mechanism for referencing all configuration values throughout HTFeed.
Helpful methods for getting config values can be found in main class modules such as METS, Volume, and VolumeValidator.

=head2 METHODS

=over 4

=item get_config()

Retrieves a value from a config file

Use get_config to reference a configuration value:

    use HTFeed::Config qw(get_config);
    get_config('staging' => 'ingest');
    get_config('jhove');


=item set_config()

Use set_config to (re)set a config entry after the config file has been loaded.

B<NOTE:> This method should be used for testing purposes only.

    use HTFeed::Config qw(set_config);
    set_config('setting','path'=>'to'=>'my'=>'setting');

=item init()

Loads the appropriate configuration files

=back

=head1 AUTHOR

    Library IT - Core Services
    University of Michigan
    lit-cs-ingest@umich.edu
    http://www.lib.umich.edu/

=head1 COPYRIGHT

Copyright (c) 2010-2012 The Regents of the University of Michigan. All rights
reserved. This program is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.



=cut
