#!/usr/bin/perl

package HTFeed::METS;
use strict;
use warnings;
use METS;
use PREMIS;
use HTFeed::XMLNamespaces qw(:namespaces :schemas);
use Carp;
use Log::Log4perl qw(get_logger);
use Time::localtime;
use HTFeed::Config qw(get_config);
use Date::Manip;
use File::Basename qw(basename dirname);
use FindBin;
use HTFeed::Version;

use base qw(HTFeed::Stage);

sub new {
    my $class = shift;

    my $self = $class->SUPER::new(
        @_,

        #		files			=> [],
        #		dir			=> undef,
        #		mets_name		=> undef,
        #		mets_xml		=> undef,
    );
    $self->{outfile} = $self->{volume}->get_mets_path();
    # by default use volume "get_pagedata" to apply pagedata
    $self->{pagedata} = sub { $self->{volume}->get_page_data(@_); };

    return $self;
}

sub run {
    my $self = shift;
    my $mets = new METS( objid => $self->{volume}->get_identifier() );
    $self->{'mets'}    = $mets;
    $self->{'amdsecs'} = [];

    $self->_add_schemas();
    $self->_add_header();
    $self->_add_dmdsecs();
    $self->_add_techmds();
    $self->_add_filesecs();
    $self->_add_struct_map();
    $self->_add_premis();
    $self->_add_amdsecs();
    $self->_save_mets();
    $self->_validate_mets();
    $self->_set_done();

}

sub stage_info {
    return { success_state => 'metsed', failure_state => 'punted' };
}

sub _add_schemas {
    my $self = shift;
    my $mets = $self->{mets};

    $mets->add_schema( "PREMIS", NS_PREMIS, SCHEMA_PREMIS );

}

sub _add_header {
    my $self = shift;
    my $mets = $self->{mets};

    my $header = new METS::Header(
        createdate   => _get_createdate(),
        recordstatus => 'NEW',
        id           => 'HDR1',
    );
    $header->add_agent(
        role => 'CREATOR',
        type => 'ORGANIZATION',
        name => get_config('mets_header_agent_name'),
    );

    $mets->set_header($header);

    return $header;
}

sub _add_dmdsecs {
    my $self   = shift;
    my $volume = $self->{volume};
    my $mets   = $self->{mets};

    my $dmdsec =
      new METS::MetadataSection( 'dmdSec',
        'id' => $self->_get_subsec_id("DMD") );
    $dmdsec->set_md_ref(
        mdtype       => 'MARC',
        loctype      => 'OTHER',
        otherloctype => 'Item ID stored as second call number in item record',
        xptr         => $volume->get_identifier()
    );
    $mets->add_dmd_sec($dmdsec);

}

# no techmds by default
sub _add_techmds {
    my $self = shift;
}

# extract existing PREMIS events from object currently in repos
sub _extract_old_premis {
    my $self   = shift;
    my $volume = $self->{volume};

    my $mets_in_repos = $volume->get_repository_mets_path();
    my $old_events    = {};

    if ( defined $mets_in_repos ) {

        # validate METS in repository
        my ( $mets_in_rep_valid, $val_results ) =
          $self->validate_xml($mets_in_repos);
        if ($mets_in_rep_valid) {
            my $xc = $volume->get_repository_mets_xpc();

            foreach my $event ( $xc->findnodes('//premis:event') ) {

                my $eventinfo = { 
                    eventtype => $xc->findvalue( "./premis:eventType", $event ) ,
                    eventid => $xc->findvalue( "./premis:eventIdentifier/premis:eventIdentifierValue", $event ),
                    eventidtype => $xc->findvalue(" ./premis:eventIdentifier/premis:eventIdentifierType", $event),
                    date => $xc->findvalue( "./premis:eventDateTime", $event ),
                };

                foreach my $field (qw(eventtype eventid date)) {
                    $self->set_error(
                        "MissingField",
                        field => "$field",
                        node  => $event->toString()
                    ) unless defined $eventinfo->{$field} and $eventinfo->{$field};
                }

                my $uuid = $volume->make_premis_uuid($eventinfo->{eventtype},$eventinfo->{date});
                my $update_eventid = 0;
                if($eventinfo->{eventidtype} ne 'UUID') {
                    get_logger()->info("Updating old event ID type $eventinfo->{eventidtype} to UUID for $eventinfo->{eventtype}/$eventinfo->{date}");
                    $update_eventid = 1;
                } elsif($eventinfo->{eventid} ne $uuid) {
                    get_logger()->warn("Warning: calculated UUID for $eventinfo->{eventtype} on $eventinfo->{date} did not match saved UUID; updating.");
                    $update_eventid = 1;
                }

                if($update_eventid) {
                    my $eventidval_node = ($xc->findnodes("./premis:eventIdentifier/premis:eventIdentifierValue",$event))[0];
                    $eventidval_node->removeChildNodes();
                    $eventidval_node->appendText($uuid);
                    my $eventidtype_node = ($xc->findnodes("./premis:eventIdentifier/premis:eventIdentifierType",$event))[0];
                    $eventidtype_node->removeChildNodes();
                    $eventidtype_node->appendText('UUID');

                }

                $old_events->{$uuid} = $event;
            }

            return $old_events;

        }
        else {

            # TODO: should be warning, not error
            $self->set_error(
                "BadFile",
                file   => $mets_in_repos,
                detail => $val_results
            );
            print "$val_results";
            return 0;
        }
    }
}

sub _add_premis_events {
    my $self            = shift;
    my $events          = shift;
    my $premis          = $self->{premis};
    my $volume          = $self->{volume};
    my $nspkg           = $volume->get_nspkg();

    EVENTCODE: foreach my $eventcode ( @{$events} ) {

        # query database for: datetime, outcome
        my $eventconfig = $nspkg->get_event_configuration($eventcode);
        my ( $eventid, $datetime, $outcome,$custom ) =
          $volume->get_event_info($eventcode);
        if(defined $custom) {
            $premis->add_event($custom);
        } else {
            $eventconfig->{eventid} = $eventid;
            $eventconfig->{date} = $datetime;
            if(defined $outcome) {
                $eventconfig->{outcomes} = [$outcome];
            }
            $self->add_premis_event($eventconfig);
        }

    }

}

sub add_premis_event {
    my $self = shift;
    my $eventconfig = shift;
    my $volume = $self->{volume};
    my $premis = $self->{premis};
    my $included_events = $self->{included_events};

    foreach my $field ('executor','executor_type','detail','type','date','eventid') {
        if(not defined $eventconfig->{$field}) {
            $self->set_error("MissingField",
                field => $field,
		actual => $eventconfig
            );
            return;
        }
    }
    
    # make sure we haven't already added this event
    my $eventid = $eventconfig->{'eventid'};
    if (defined $included_events->{$eventid}) {
        return;
    } 

    my $event = new PREMIS::Event( $eventconfig->{'eventid'}, 'UUID', 
                                   $eventconfig->{'type'}, $eventconfig->{'date'},
                                   $eventconfig->{'detail'});
    foreach my $outcome (@{ $eventconfig->{'outcomes'} }) {
        $event->add_outcome($outcome);
    }

# query namespace/packagetype for software tools to record for this event type
    $event->add_linking_agent(
        new PREMIS::LinkingAgent( $eventconfig->{'executor_type'}, 
                                  $eventconfig->{'executor'}, 
                                  'Executor' ) );

    my @agents       = ();
    my $tools_config = $eventconfig->{'tools'};
    foreach my $agent (@$tools_config) {
        $event->add_linking_agent(
            new PREMIS::LinkingAgent( 'tool', $self->get_tool_version($agent), 'software')
        );
    }
    $included_events->{$eventid} = $event;
    $premis->add_event($event);

    return $event;
}

# Baseline source METS extraction for cases where source METS PREMIS events
# need no modification for inclusion into HT METS

sub _add_source_mets_events {
    my $self   = shift;
    my $volume = $self->{volume};
    my $premis = $self->{premis};

    my $xc                = $volume->get_source_mets_xpc();
    my $src_premis_events = {};
    foreach my $src_event ( $xc->findnodes('//premis:event') ) {

        # src event will be an XML node
        # do we want to keep this kind of event?
        my $event_type = $xc->findvalue( './premis:eventType', $src_event );
        $src_premis_events->{$event_type} = []
          if not defined $src_premis_events->{$event_type};
        push( @{ $src_premis_events->{$event_type} }, $src_event );
    }

    foreach my $eventcode (
        @{ $volume->get_nspkg()->get('source_premis_events_extract') } )
    {
        my $eventconfig = $volume->get_nspkg()->get_event_configuration($eventcode);
        my $eventtype = $eventconfig->{type};
        next unless defined $src_premis_events->{$eventtype};
        foreach my $src_event ( @{ $src_premis_events->{$eventtype} } ) {
            my $eventid = $xc->findvalue( "./premis:eventIdentifier[premis:eventIdentifierType='UUID']/premis:eventIdentifierValue",
                $src_event
            );
            if ( not defined $self->{included_events}{$eventid} ) {
                $self->{included_events}{$eventid} = $src_event;
                $premis->add_event($src_event);
            }
        }
    }
}

sub _add_premis {
    my $self   = shift;
    my $volume = $self->{volume};
    my $nspkg  = $volume->get_nspkg();

    # map from UUID to event - events that have already been added
    $self->{included_events} = {};

    my $premis = new PREMIS;
    $self->{premis} = $premis;

    my $old_events = $self->_extract_old_premis();
    if ($old_events) {
        while ( my ( $eventid, $event ) = each(%$old_events) ) {
            $self->{included_events}{$eventid} = $event;
            $premis->add_event($event);
        }
    }

    $self->_add_source_mets_events();

    # create PREMIS object
    my $premis_object =
      new PREMIS::Object( 'identifier', $volume->get_identifier() );
    $premis_object->set_preservation_level("1");
    $premis_object->add_significant_property( 'file count',
        $volume->get_file_count() );
    if ($volume->get_file_groups()->{image}) {
        $premis_object->add_significant_property( 'page count',
            $volume->get_page_count() );
    }
    $premis->add_object($premis_object);

    # last chance to record, even though it's not done yet
    $volume->record_premis_event('ingestion');

    $self->_add_premis_events( $nspkg->get('premis_events') );

    my $digiprovMD =
      new METS::MetadataSection( 'digiprovMD', 'id' => 'premis1' );
    $digiprovMD->set_xml_node( $premis->to_node(), mdtype => 'PREMIS' );

    push( @{ $self->{amd_mdsecs} }, $digiprovMD );

}

sub _add_amdsecs {
    my $self = shift;
    $self->{'mets'}
      ->add_amd_sec( $self->_get_subsec_id("AMD"), @{ $self->{amd_mdsecs} } );

}

sub _get_subsec_id {
    my $self        = shift;
    my $subsec_type = shift;
    $self->{counts} = {} if not exists $self->{counts};
    $self->{counts}{$subsec_type} = 0
      if not exists $self->{counts}{$subsec_type};
    return "$subsec_type" . ++$self->{counts}{$subsec_type};
}

sub _add_zip_fg {
    my $self   = shift;
    my $mets   = $self->{mets};
    my $volume = $self->{volume};

    $volume->record_premis_event('zip_md5_create');
    my $zip_filegroup = new METS::FileGroup(
        id  => $self->_get_subsec_id("FG"),
        use => 'zip archive'
    );
    my ($zip_path,$zip_name) = ($volume->get_zip_directory(), $volume->get_zip_filename());
    $zip_filegroup->add_file( $zip_name, path => $zip_path, prefix => 'ZIP' );
    $mets->add_filegroup($zip_filegroup);
}

sub _add_srcmets_fg {
    my $self   = shift;
    my $mets   = $self->{mets};
    my $volume = $self->{volume};

    # Add source METS if it is present
    my $src_mets_file = $self->{volume}->get_source_mets_file();

    if($src_mets_file) {
        my $mets_filegroup = new METS::FileGroup(
            id  => $self->_get_subsec_id("FG"),
            use => 'source METS'
        );
        $mets_filegroup->add_file( $src_mets_file, 
            path => $volume->get_staging_directory(), 
            prefix => 'METS' );
        $mets->add_filegroup($mets_filegroup);
    }
}

sub _add_content_fgs {
    my $self   = shift;
    my $mets   = $self->{mets};
    my $volume = $self->{volume};

    # then add the actual content files
    my $filegroups = $volume->get_file_groups();
    $self->{filegroups} = {};
    while ( my ( $filegroup_name, $filegroup ) = each(%$filegroups) ) {
        my $mets_filegroup = new METS::FileGroup(
            id  => $self->_get_subsec_id("FG"),
            use => $filegroup->get_use()
        );
        $mets_filegroup->add_files( $filegroup->get_filenames(),
            prefix => $filegroup->get_prefix(),
            path => $volume->get_staging_directory() );

        $self->{filegroups}{$filegroup_name} = $mets_filegroup;
        $mets->add_filegroup($mets_filegroup);
    }
}

sub _add_filesecs {
    my $self = shift;

    # first add zip
    $self->_add_zip_fg();
    $self->_add_srcmets_fg();
    $self->_add_content_fgs();

}

# Basic structMap with optional page labels.
sub _add_struct_map {
    my $self   = shift;
    my $mets   = $self->{mets};
    my $volume = $self->{volume};
    my $get_pagedata = $self->{pagedata};

    my $struct_map = new METS::StructMap( id => 'SM1', type => 'physical' );
    my $voldiv = new METS::StructMap::Div( type => 'volume' );
    $struct_map->add_div($voldiv);
    my $order               = 1;
    my $file_groups_by_page = $volume->get_structmap_file_groups_by_page();
    foreach my $seqnum ( sort( keys(%$file_groups_by_page) ) ) {
        my $pagefiles   = $file_groups_by_page->{$seqnum};
        my $pagediv_ids = [];
        my $pagedata;
        my @pagedata;
        while ( my ( $filegroup_name, $files ) = each(%$pagefiles) ) {
            foreach my $file (@$files) {
                my $fileid =
                  $self->{filegroups}{$filegroup_name}->get_file_id($file);
                if ( not defined $fileid ) {
                    $self->set_error(
                        "MissingField",
                        field     => "fileid",
                        file      => $file,
                        filegroup => $filegroup_name,
                        detail    => "Can't find ID for file in file group"
                    );
                    next;
                }

                if(defined $get_pagedata) {
                    # try to find page number & page tags for this page
                    if ( not defined $pagedata ) {
                        $pagedata = &$get_pagedata($file);
                        @pagedata = %$pagedata if defined $pagedata;
                    }
                    else {
                        my $other_pagedata = &$get_pagedata($file);
                        while ( my ( $key, $val ) = each(%$pagedata) ) {
                            my $val1 = $other_pagedata->{$key};
                            $self->set_error(
                                "NotEqualValues",
                                actual => "other=$val ,$fileid=$val1",
                                detail =>
    "Mismatched page data for different files in pagefiles"
                              )
                              unless ( not defined $val and not defined $val1 )
                              or ( $val eq $val1 );
                        }

                    }
                }

                push( @$pagediv_ids, $fileid );
            }
        }
        $voldiv->add_file_div(
            $pagediv_ids,
            order => $order++,
            type  => 'page',
            @pagedata
        );
    }
    $mets->add_struct_map($struct_map);

}

sub _save_mets {
    my $self = shift;
    my $mets = $self->{mets};

    my $mets_path = $self->{outfile};

    open( my $metsxml, ">", "$mets_path" )
      or die("Can't open METS xml $mets_path for writing: $!");
    print $metsxml $mets->to_node()->toString(1);
    close($metsxml);
}

sub _validate_mets {
    my $self      = shift;
    my $mets_path = $self->{outfile};

    croak("File $mets_path does not exist. Cannot validate.")
      unless -e $mets_path;

    my ( $mets_valid, $val_results ) = $self->validate_xml($mets_path);
    if ( !$mets_valid ) {
        $self->set_error(
            "BadFile",
            file   => $mets_path,
            detail => "XML validation error: $val_results"
        );

        # TODO: set failure creating METS file
        return;
    }

}

sub validate_xml {
    my $self   = shift;
    my $use_caching = $self->{volume}->get_nspkg()->get('use_schema_caching');
    my $schema_cache = get_config('xerces_cache');
    my $xerces = get_config('xerces');

    $xerces .= " $schema_cache" if($use_caching);

    my $filename       = shift;
    my $validation_cmd = "$xerces '$filename' 2>&1";
    my $val_results    = `$validation_cmd`;
    if ( ($use_caching and $val_results !~ /\Q$filename\E OK/) or
         (!$use_caching and $val_results =~ /Error/) or
        $? ) {
        wantarray ? return ( 0, $val_results ) : return (0);
    }
    else {
        wantarray ? return ( 1, undef ) : return (0);
    }

}

# Given ss1970, use Time::localtime to generate a date with format: yyyy-mm-ddT13:27:00
sub _get_createdate {
    my $self   = shift;
    my $ss1970 = shift;

    my $localtime_obj = defined($ss1970) ? localtime($ss1970) : localtime();

    my $ts = sprintf(
        "%d-%02d-%02dT%02d:%02d:%02d",
        ( 1900 + $localtime_obj->year() ), ( 1 + $localtime_obj->mon() ),
        $localtime_obj->mday(), $localtime_obj->hour(),
        $localtime_obj->min(),  $localtime_obj->sec()
    );

    return $ts;
}

sub perl_mod_version() {
    my $self    = shift;
    my $module  = shift;
    my $mod_req = $module;
    $mod_req =~ s/::/\//g;
    my $toreturn;
    eval { require "$mod_req.pm"; };
    if ($@) {
        $self->set_error( 'ToolVersionError',
            detail => "Error loading $module: $@" );
        return "$module";
    }
    no strict 'refs';
    my $version = ${"${module}::VERSION"};
    if ( defined $version ) {
        return "$module $version";
    }
    else {
        $self->set_error( 'ToolVersionError',
            detail => "Can't find ${module}::VERSION" );
        return "$module";
    }
}

sub local_directory_version() {
    my $self      = shift;
    my $package   = shift;
    my $tool_root = get_config("premis_tool_local");
    if ( not -l "$tool_root/$package" ) {
        $self->set_error( 'ToolVersionError',
            detail => "$tool_root/$package not a symlink" );
        return $package;
    }
    else {
        my $package_target;
        if ( !( $package_target = readlink("$tool_root/$package") ) ) {
            $self->set_error( 'ToolVersionError',
                detail => "Error in readlink for $tool_root/$package: $!" )
              if $!;
            return $package;
        }

        my ($package_version) = ( $package_target =~ /$package-([\w.]+)$/ );
        if ($package_version) {
            return "$package $package_version";
        }
        else {
            $self->set_error( 'ToolVersionError',
                detail => "Couldn't extract version from symlink $package_version for $package");
            return $package;
        }

    }
}

sub system_version() {
    my $self    = shift;
    my $package = shift;
    my $version = `rpm -q $package`;

    if ( $? or $version !~ /^$package[-.\w]+/ ) {
        $self->set_error( 'ToolVersionError',
            detail =>
              "RPM returned '$version' with status $? for package $package" );
        return $package;
    }
    else {
        chomp $version;
        return $version;
    }
}

sub get_tool_version {

    my $self       = shift;
    my $package_id = shift;
    my $to_eval    = get_config( 'premis_tools', $package_id );
    if ( !$to_eval ) {
        $self->set_error( 'ToolVersionError',
            detail => "$package_id missing from premis_tools" );
        return $package_id;
    }

    my $version = eval($to_eval);
    if ( $@ or !$version ) {
        $self->set_error( 'ToolVersionError', detail => $@ );
        return $package_id;
    }
    else {
        return $version;
    }
}

sub clean_always {
    my $self = shift;

    $self->{volume}->clean_unpacked_object();
}

# do cleaning that is appropriate after failure
sub clean_failure {
    my $self = shift;
    $self->{volume}->clean_mets();
}

# fixes errors in MARC leader by changing fields to their default value if they
# do not match the regular expression for the leader in the MARC schema
sub _remediate_marc {
    my $self = shift;
    my $xc = shift;

    my @leaders = $xc->findnodes(".//marc:leader");
    if(@leaders != 1) {
        $self->set_error("BadField",field=>"marc:leader",detail=>"Zero or more than one leader found");
        return;
    }

    my $leader = $leaders[0];
    my $value = $leader->findvalue(".");

    if ($value !~ /^
        [\d ]{5} # 00-04: Record length
        [\dA-Za-z ]{1} # 05: Record status
        [\dA-Za-z]{1} # 06: Type of record
        [\dA-Za-z ]{3} # 07: Bibliographic level
                       # 08: Type of control
                       # 09: Character
        (2| )          # 10: Indicator count
        (2| )          # 11: Subfield code count
        [\d ]{5}       # 12: Base address of data
        [\dA-Za-z ]{3} # 17: Encoding level
                       # 18: Descriptive cataloging form
                       # 19: Multipart resource record level
        (4500|    )    # 20: Length of the length-of-field portion
                       # 21: Length of the starting-character-position portion
                       # 22: Length of the implementation-defined portion
                       # 23: Undefined
        $/x) {

        # 00-04: Record length - default to empty
        if(substr($value,0,5) !~ /^[\d ]{5}$/) {
            substr($value,0,5) = '     ';
        }

        # 05: Record status
        if(substr($value,5,1) !~ /^[\dA-Za-z ]$/) {
            substr($value,5,1) = ' ';
        }

        # 06: Type of record
        if(substr($value,6,1) !~ /^[\dA-Za-z]$/) {
            get_logger()->warn("Invalid value found for record type, can't remediate");
        }

        # 07: Bibliographic level
        if(substr($value,7,1) !~ /^[\dA-Za-z ]$/) {
            substr($value,7,1) = ' ';
        }

        # 08: Type of control
        if(substr($value,8,1) !~ /^[\dA-Za-z ]$/) {
            substr($value,8,1) = ' ';
        }

        # 09: Character coding scheme
        if(substr($value,9,1) ne 'a') {
            get_logger()->warn("Non-Unicode MARC-XML found");
        }

        # 10: Indicator count
        if(substr($value,10,1) !~ /^(2| )$/) {
            substr($value,10,1) = ' ';
        }

        # 11: Subfield code count
        if(substr($value,11,1) !~ /^(2| )$/) {
            substr($value,11,1) = ' ';
        }

        # 12-16: Base address of data
        if(substr($value,12,5) !~ /^[\d ]{5}$/) {
            substr($value,12,5) = '     ';
        }

        # 17: Encoding level
        if(substr($value,17,1) !~ /^[\dA-Za-z ]$/) {
            substr($value,17,1) = 'u'; # unknown
        }

        # 18: Descriptive cataloging form
        if(substr($value,18,1) !~ /^[\dA-Za-z ]$/) {
            substr($value,18,1) = 'u'; # unknown
        }

        # 19: Multipart resource record level
        if(substr($value,19,1) !~ /^[\dA-Za-z ]$/) {
            substr($value,19,1) = ' '; 
        }

        # 20: Length of the length-of-field portion
        # 21: Length of the start-character-position portion
        # 22: Length of the implementatino-defined portion
        # 23: Undefined
        if(substr($value,20,4) !~ /^(4500|    )/) {
            # default to unspecified
            substr($value,20,4) = '    ';
        }

        $leader->removeChildNodes();
        $leader->appendText($value);
    }

    foreach my $datafield ($xc->findnodes('./marc:datafield')) {
        # ind1/ind2 might have nbsp or control characters instead of regular space
        # @i1 => @ind1
        # @i2 => @ind2
        my $attrs_to_move = {
            # clean ind1, ind2; move i{1,2} -> ind{1,2}
            'ind1' => 'ind1',
            'ind2' => 'ind2',
            'i1' => 'ind1',
            'i2' => 'ind2',
        };
        while (my ($old,$new) = each (%$attrs_to_move)) {
            if($datafield->hasAttribute($old)) {

                my $attrval = $datafield->getAttribute($old);
                # default to empty if value is invalid
                if($attrval !~ /^[\da-z ]{1}$/) {
                    $attrval = " ";
                }
                $datafield->removeAttribute($old);
                $datafield->setAttribute($new,$attrval);
            }
        }
    }

    foreach my $datafield ($xc->findnodes('.//marc:datafield[not(marc:subfield)]')) {
        # remove empty data fields
        $datafield->parentNode()->removeChild($datafield);
    }


}

1;

__END__

=head1 NAME

HTFeed::METS - Main class for creating METS XML

=head1 SYNOPSIS

A series of stages to generate a METS XML document for a Feed package.

=head1 DESCRIPTION 

METS.pm provides the main methods for generating a METS XML document.
These methods (documented below) can be subclassed for various special cases, such as SourceMETS and PackageType::METS.

=head2 METHODS

=over 4

=item new()

Instantiate the class

=item run()

Run a series of internally defined stages to generate the METS elements:

schemas

header

dmdsecs

techmds 

filesecs

struct_map

premis

amdsecs

The run() method also validates and saves the METS XML document.

=item perl_mod_version()

Return $module::VERSION; B<NOTE:> $module must have already been loaded.

C<$version = perl_mod_version($module);>

=item stage_info()

Return status on completion of METS stage (success/failure) 

=item add_premis_event()

Add a PREMIS event

=item local_directory_version()

Return the version of a package installed in a local directory hierarchy,
specified by the 'premis_tool_local' configuration directive

$local_version = local_directory_version($package);

=item system_version()

Return the version of a system-installed RPM package.

$package_version = system_version($package);

=item get_tool_version()

Get the version of a tool defined in the premis_tools section in the configuration file.

$tool_version = get_tool_version($package_identifier);

=item clean_always()

Do the cleaining that is appropriate for this stage.

=item validate_xml()

Validate the METS XML against the defined schema.

=item clean_failure()

Do the cleaning that is appropriate on stage failure.

=back

=head1 AUTHOR

    Library IT - Core Services
    University of Michigan
    lit-cs-ingest@umich.edu
    http://www.lib.umich.edu/

=head1 COPYRIGHT

Copyright (c) 2010-2012 The Regents of the University of Michigan. All rights
reserved. This program is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.



=cut
