package HTFeed::PackageType::Google;

use warnings;
use strict;
use base qw(HTFeed::PackageType);

our $identifier = 'google';

our $config = {

    %{$HTFeed::PackageType::config},
    description => 'Google-digitized book content',

    volume_module => 'HTFeed::PackageType::Google::Volume',

    # google stuff doesn't start off ready to download, you have to
    # go through the conversion song & dance. so the default add 
    # state is 'available' instead of 'ready'.
    default_queue_state => 'available',

    # Regular expression that distinguishes valid files in the file package
    # HTML OCR is valid for the package type but only expected/required for UC1
    valid_file_pattern => qr/^( 
    checksum\.md5 |
    \w+.(xml) |
    \d{8}.(html|jp2|tif|txt)
    )/x,

    # Configuration for each filegroup. 
    # prefix: the prefix to use on file IDs in the METS for files in this filegruop
    # use: the 'use' attribute on the file group in the METS
    # file_pattern: a regular expression to determine if a file is in this filegroup
    # required: set to 1 if a file from this filegroup is required for each page 
    # content: set to 1 if file should be included in zip file
    # jhove: set to 1 if output from JHOVE will be used in validation
    # utf8: set to 1 if files should be verified to be valid UTF-8
    filegroups => {
        image => { 
            prefix => 'IMG',
            use => 'image',
            file_pattern => qr/\d{8}\.(jp2|tif)$/,
            required => 1,
            content => 1,
            jhove => 1,
            utf8 => 0
        },
        ocr => { 
            prefix => 'TXT',
            use => 'ocr',
            file_pattern => qr/\d{8}\.txt$/,
            required => 1,
            content => 1,
            jhove => 0,
            utf8 => 1
        },
        hocr => { 
            prefix => 'HTML',
            use => 'coordOCR',
            file_pattern => qr/\d{8}\.html$/,
            required => 1,
            content => 1,
            validate => 1,
            jhove => 0,
            utf8 => 1
        }
    },

    checksum_file => qr/checksum\.md5$/,
    source_mets_file => qr/\w+\.xml$/,

    # what stage to run given the current state
    stage_map => {
        ready      => 'HTFeed::PackageType::Google::Download',
        downloaded => 'HTFeed::PackageType::Google::Unpack',
        unpacked   => 'HTFeed::VolumeValidator',
        validated  => 'HTFeed::Stage::Pack',
        packed     => 'HTFeed::PackageType::Google::METS_PREMIS1',
        metsed     => 'HTFeed::Stage::Handle',
        handled    => 'HTFeed::Stage::Collate',
    },


    # What PREMIS events to include (by internal PREMIS identifier, 
    # configured in config.yaml)
    premis_events => [
    'decryption',
    'page_md5_fixity',
    'package_validation',
    'zip_compression',
    'zip_md5_create',
    #	'ht_mets_creation',
    'ingestion',
    ],

    # Overrides for the basic PREMIS event configuration
    premis_overrides => {
        'decryption' => {
            type => 'decryption',
            detail => 'Decryption of Google archive',
            executor => 'MiU',
            executor_type => 'MARC21 Code',
            tools => [qw(GPG)]
        },
        'image_compression' => {
            executor => 'Ca-MvGOO',
            executor_type => 'MARC21 Code'
        }
    },

    SIP_filename_pattern => '%s.tar.gz.gpg',

    validation => {
        'HTFeed::ModuleValidator::JPEG2000_hul' => {
            'decomposition_levels' => sub {
                my $self = shift;

                # In some cases Google will return a 1-pixel JPEG2000 image as a placeholder for
                # a missing page, and it will always have decompositionLevels = 1. This is OK!

                my $decomp_levels = $self->_findone('codingStyleDefault','decompositionLevels');
                my $x1 = $self->_findone( "mix", "width" );
                my $y1 = $self->_findone( "mix", "length" );
                if($x1 == 1 and $y1 == 1) {
                    if($decomp_levels == 1) {
                        return 1;
                    } else {
                        $self->set_error("BadValue",field => "decomposition_levels",actual=>$decomp_levels,expected=>1);
                    }
                } else {
                    if($decomp_levels >= 5 and $decomp_levels <= 32) {
                        return 1;
                    } else {
                        $self->set_error("BadValue",field => "decomposition_levels",actual=>$decomp_levels,expected=>"5 <= decomp_levels <= 32");
                    }
                }

            }
        }
    }

};

__END__

=pod

This is the package type configuration file for Google / GRIN.

=head1 SYNOPSIS

use HTFeed::PackageType;

my $pkgtype = new HTFeed::PackageType('google');

=head1 AUTHOR

Aaron Elkiss, University of Michigan, aelkiss@umich.edu

=head1 COPYRIGHT

Copyright (c) 2010 University of Michigan. All rights reserved.  This program
is free software; you can redistribute it and/or modify it under the same terms
as Perl itself.

=cut
