package HTFeed::Stage::Fetch;

use strict;
use warnings;
use base qw(HTFeed::Stage::DirectoryMaker);
use Log::Log4perl qw(get_logger);
use HTFeed::Config qw(get_config);

=head1 NAME

HTFeed::Stage::Fetch

=item DESCRIPTION

Base class for HTFeed Fetch stage
Fetches new material for ingest from a networked location

=cut

=item fetch_from_source()

 fetch_from_source($source, $destination)
 fetches a package from directory $source
 and deposits it in directory $destination

=cut

sub fetch_from_source {

	my $self = shift;
	my $source = shift;
	my $dest = shift;

	if(! -e $dest) {
		mkdir $dest or die("Can't mkdir $dest $!");
	}
	
    get_logger()->trace("Fetching from source: cp -Lrs '$source' '$dest'");
	system("cp -Lrs '$source' '$dest'")
        and $self->set_error('OperationFailed', operation=>'copy', detail=>"copy $source $dest failed with status: $?");
}

=item fix_line_endings()

fix_line_endings($destination)
Fixes line endings in text files
in directory $destination

=cut

sub fix_line_endings {
    my $self = shift;
    my $base = shift;
	my $volume = $self->{volume};
	my $objid = $volume->get_objid();

	my $dir = "$base/$objid";

    foreach my $filename (glob("$dir/*.txt"), "$dir/checksum.md5", "$dir/pageview.dat") {
        next unless -e $filename;

        if( -e "$filename.bak" ) {
            $self->set_error("UnexpectedError","$filename.bak exists");
        }
        
        rename("$filename","$filename.bak") or $self->set_error("OperationFailed",operation => "rename",file => $filename,detail => $!);
        open INPUT, "$filename.bak" or $self->set_error("OperationFailed",operation => "open", file => "$filename.bak", detail => $!);
        open OUTPUT, ">$filename" or $self->set_error("OperationFailed",operation => "open", file => "$filename", detail => $!);

        while( <INPUT> ) {
            s/\r\n$/\n/;     # convert CR LF to LF
            print OUTPUT $_;
        }

        close INPUT;
        close OUTPUT;
        unlink("$filename.bak") or $self->set_error("OperationFailed",operation => "unlink", file => "$filename.bak", detail => $!);
        get_logger()->trace("Cleaned line endings for $filename");

    }

}

=item stage_info()

Returns stage data based on success/failure

=cut

sub stage_info{
	return {success_state => 'fetched', failure_state => 'punted'};
}

1;

__END__

=pod

=head1 AUTHOR

    Library IT - Core Services
    University of Michigan
    lit-cs-ingest@umich.edu
    http://www.lib.umich.edu/

=head1 COPYRIGHT

Copyright (c) 2010-2012 The Regents of the University of Michigan. All rights
reserved. This program is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.



=cut
